/* @(#)fsyunix.c	16.1.1.1 (ESO-DMD) 06/19/01 15:18:48 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*++++++++++++++++++++++++++ Module FSY ++++++++++++++++++++++++++++++++
.TYPE Module
.NAME FSY
.LANGUAGE C
.AUTHOR
  K. Banse                              ESO - Garching
.CATEGORY Image files handling.
.COMMENTS 
 holds FSY_EXTBDF, FSY_MODBDF

.VERSION 

 010425		last modif

---------------------------------------------------------------------------*/

#include  <fileexts.h>
#include  <errno.h>
#include  <sys/types.h>   /* to be used in extbdf !!! temporarily */
#include  <sys/stat.h>
#include  <osyparms.h>
#include  <osparms.h>


static char   block[BLOCK_SIZE];
static struct stat buf;
static int first = 0;
static long int  filsize;

/*

*/

void FSY_EXTBDF(fileid, exalq, nvb, status)
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE  Extend Bulk Data File
.METHOD This routine is used to extend an EXISTING Bulk Data File. The
	file is opened using osdseek and osdwrite; the calling program supplies
	the  file identification and the number of bytes to write.
	(i.e. it is assumed that the file has been opened previously)
	If the file is opened successfully, the initial file size
	(measured in virtual blocks) is saved and the file extended
	by the specified number of blocks; the new file size is then
	calculated and returned to the calling program.
.RETURNS nothing
-------------------------------------------------------------------------*/
int   fileid         /* file identification */;
int   exalq          /* extension allocation quantity */;
int   *nvb           /* number of virtual blocks actually allocated */;
int   *status        /* return status (1 if correct, else 0!)  */;

{
int  jstat;
register int i;
unsigned int nblocksz;



if (first == 0)
   {
   for (i=0; i<BLOCK_SIZE; i++) block[i] = '\0';
   first = 1;
   }


/* the file is supposed to have previously been opened */

nblocksz = BLOCK_SIZE;
*status = 1;				/* default to success */


/* seek to the end of file */
    
jstat = osdseek(fileid,0L,FILE_END);
if (jstat == -1) goto bad_return;
    

/* write exalq * BLOCK_SIZE bytes at this location */

for (i=0; i<exalq; i++)
   {
   jstat = osdwrite(fileid,block,nblocksz);
   if (jstat < BLOCK_SIZE) goto bad_return;
   }

fstat(fileid,&buf);		/* get new file size */
filsize = buf.st_size;
filsize /= BLOCK_SIZE;
*nvb = (int) filsize;
return;


bad_return:
*status = 0;
}

/*
^L
*/

void FSY_OPNFIL(name, len, iochan, status)
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE Open Binary File
.METHOD
        This routine is used to open an EXISTING Data File for
        subsequent processing. The filename, logical or physical, is
        supplied by the calling program and the file is opened by
        the osmopen interface facility.
.RETURNS nothing
-------------------------------------------------------------------------*/

char  *name           /* file name */;
int   len             /* length of file name, not used in Unix!  */;
int   *iochan         /* iochannel (file identifier in UNIX) */;
int   *status         /* return status (1 if correct, 0 elsewhere )!**/;
   
{

int  filid;


filid = osmopen(name, READ_WRITE);


/* if no permission for attempted operation, try to open it for read only */

if (filid == -1) filid = osmopen(name,READ);


*iochan = filid;
if (filid < 0)              /* open has failed anyway */
   *status = 0;
else
   *status = 1;

}

/*

*/

int FSY_MODBDF(fileid, nobyt, nvb)
int   fileid;		/* file identification */
long int nobyt;		/* no. (position) of byte to be written */
int   *nvb;		/* number of blocks allocated for file*/

{

#ifdef __FreeBSD__
  off_t lseek();
#else
  long int lseek();
#endif 
long int  ret;

char c = ' ';



ret = lseek(fileid,nobyt-1,FILE_START);		/* position to `nobyt-1' */
if (ret == -1L) 
   {
   oserror = errno;
   close(fileid);
   return(-1);
   }

if (write(fileid,&c,1) != 1) 
   {						/* Write last byte */
   oserror = errno;
   close(fileid);
   return(-1);
   }

fstat(fileid,&buf);

filsize = buf.st_size;
filsize /= BLOCK_SIZE;
*nvb = (int) filsize;
return 0;
}

